#    uwss - uber web security scanner
#    Copyright (C) 2009 noptrix & illuminatus
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# uAttack.py
#
# by noptrix & illuminatus
# www.noptrix.net


import sys

# uwss imports
from udb.uSQL import *
from uhttp.uHttp import *
from uconfig.uConfig import *
from uattack.uCheck import *


class Attack():
   """
   attack class
   """

   def __init__(self, url_data, attack_module, request_type, opts):
      """
      constructor
      """

      # verbose and debug mode
      self.opts = opts

      # attack module
      self.attack_module = attack_module

      # url data, contains url and/or post data
      self.url_data = url_data

      # http request type: GET or POST
      self.request_type = request_type

      # set url
      if (self.request_type == 'GET'):
         self.target_url = self.url_data

      elif (self.request_type == 'POST'):
         self.target_url = self.url_data[0]

      # set http handler
      self.http_handler = httpRequest(opts)


   def get_attack_types_from_db(self):
      """
      fetch attack-types, payloads and success-data from our attacks.db
      """

      # fetch available attacks from db
      db_handler = attacksDB(self.opts['payloads_file'], self.opts)
      db_handler.connect()

      # contains attack-types, payloads and success-data
      try:
         self.attack_data = db_handler.show_attacks_by_type(self.attack_module,\
               self.opts['payloads_max'])
      except NetError, e:
         e.print_error()

      db_handler.disconnect()


   def attack(self):
      """
      send the attack (request)
      """

   
      # found bugs
      vulns = []

      # fetch all attack data (type, payloads, success-data)fg
      self.get_attack_types_from_db()

      # for GET it contains complete attack_url: url + payload
      # for POST it contains form values, which will be replaced by payloads
      values = {}

      for i in range(len(self.attack_data)):
         values = (self.build_attack_url(self.attack_data[i][2]))
         try:
            if self.request_type == 'GET':
               response = self.http_handler.send_GET_request(values)
            elif self.request_type == 'POST':
               response = \
                     self.http_handler.send_POST_request(self.url_data[0], values)
         except NetError, e:
            continue

         # check for vulnerabilities and save them
         vulnerable = None
         if response:
            check = Check()
            vulnerable = check.check_vuln(self.target_url, values, response,
                self.attack_data[i][3], self.attack_module)

         # Vulnerabilities found
         if vulnerable:
            vulns.append(values)
            sys.stdout.write('F')
         else:
            sys.stdout.write('.')
         
         sys.stdout.flush()

      return vulns


   def build_attack_url(self, payloads):
      """
      build the attack url: URL + <payload>
      """

      # build a complete attack url: url+payload
      if self.request_type == 'GET':
         if CHANGEME in self.url_data:
            attack_url = self.url_data.replace(CHANGEME, payloads)
            return attack_url
         else:
            attack_url = self.url_data + payloads
            return attack_url

      # replace %CHANGEME% by <payload>
      elif self.request_type == 'POST':
         if CHANGEME in self.url_data[1].values(): 
            tmp_dict = {}
            for i in self.url_data[1].keys():
                if self.url_data[1][i]:
                    tmp = self.url_data[1][i].replace(CHANGEME, payloads)
                    tmp_dict[i] = tmp
            return tmp_dict
         return self.url_data


# EOF
