#    uwss - uber web security scanner
#    Copyright (C) 2009 noptrix & illuminatus
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# uController.py
#
# by noptrix & illuminatus
# www.noptrix.net


# uwss imports
from uconfig.uXml import *
from ucrawler.uCrawler import *
from ucrawler.lswww import *
from uattack.uAttack import *
from uattack.uAttackDynamicKeys import *
from ureport.uReport import *
from uconfig.uConfig import *
from uhttp.uHttp import *
from uhttp.uCookie import *


class Controller():
   """
   """

   # Report data: attack types + vulnerabilities
   g_report_data = []

   def __init__(self, (options, args)):
      """
      constructor
      """

      self.options = options


   @classmethod 
   def print_banner(Controller):
      """
      print uwss banner
      """

      print UWSS_BANNER


   def get_output_mode(self):
      """
      get values for verbosity and debug options
      """

      if self.options.verbose: opts['VERBOSE'] = 1
      if self.options.debug: opts['DEBUG'] = 1


   def get_xml_config(self, (options, args)):
      """
      read and parse xml config file
      """

      self.options = options

      if self.options.filename:
         xml_stuff = xmlParser(self.options.filename, opts)
         try:
            xml_config = xml_stuff.xml_read_config()
         except ConfigError, e:
            e.print_error()

         return xml_config


   def get_auth_cookies(self, (options,args), xml_config):
      """
      Get authentification cookies from server
      """
      
      print BEGIN_AUTHENTIFICATION
      # Set request timeout
      opts['timeout'] = xml_config['target_req_timeout']
      
      # Send request and get dynamic data, like e.g. keys etc.
      req = httpRequest(opts)

      # set header options
      headers = {'User-Agent': xml_config['crawler_user_agent']}
      req.set_headers(headers)

      # Send request
      u = req.send_GET_request(xml_config['acc_url'])

      # Create linkParser instance and parse response
      p = linkParser()

      # Set excluded list of POST params
      p.excluded_post_params = xml_config['crawler_exclude_post_params']
      
      p.feed(u.read())

      # Get login form
      login_form = None
      for form in p.forms:
         if xml_config['acc_url'] in form[0]:
             login_form = form

      if not login_form:
         print "[-] Form not found..."
         return

      # Set login data
      login_form[1][xml_config['acc_field_user']] = xml_config['acc_user']
      login_form[1][xml_config['acc_field_passwd']] = xml_config['acc_passwd']

      # Set cookie handler
      req.set_cookie_handler()

      # Send POST request
      p = req.send_POST_request(login_form[0], login_form[1])

      # ... and save cookies to file
      req.cookies.save(xml_config['acc_cookie'])

      print END_AUTHENTIFICATION % xml_config['acc_cookie']


   def start_crawler(self, (options, args), xml_config):
      """
      start crawler
      """

      self.options = options

      print BEGIN_CRAWLER

      # Set request timeout
      opts['timeout'] = xml_config['target_req_timeout']

      # Set cookie file for crawler
      if os.path.isfile(xml_config['acc_cookie']):
          opts['cookie_file'] = xml_config['acc_cookie']

      # new handler for crawler class
      crawler_handler = crawler(xml_config['target_url'], opts)

      # set header options
      headers = {'User-Agent': xml_config['crawler_user_agent']}
      crawler_handler.setHeaders(headers)

      # set max deep level for crawling (in your xml config file
      crawler_handler.setMaxDeep(int(xml_config['crawler_deep_level']))
   
      # set verbosity for crawler information outputs
      crawler_handler.verbosity(int(xml_config['crawler_verbosity']))

      # set list of excluded POST parameters
      crawler_handler.set_excluded_post_param_list(xml_config['crawler_exclude_post_params'])
         
      # Set list of excluded URLs
      for i in xml_config['crawler_exclude_urls']:
          crawler_handler.addExcludedURL(i)

      # GO crawl
      crawler_handler.go()

      # preprocess the data: replace values with %CHANGEME%
      crawler_handler.preprocess_links()

      # save the results to specified destination 
      crawler_handler.save_data(xml_config['crawler_website_data'])

      print END_CRAWLER % \
          (len(crawler_handler.get_links()), len(crawler_handler.get_forms()))

      # only in verbose mode!
      if options.verbose:
          import pprint
          pprint.pprint(crawler_handler.get_forms())

      sys.exit(0)


   def get_links_and_forms(self, xml_config):
      """
      get links from website
      """

      # new Crawler instance
      crawler_handler = crawler(xml_config['target_url'], opts)

      # read whole website data from file
      crawler_handler.read_data(xml_config['crawler_website_data'])

      # get the links and form data
      links = crawler_handler.get_links()
      forms = crawler_handler.get_forms()

      return links, forms


   def send_attacks(self, module, data, req_type):
      """
      send attacks
      """

      # Timestamps difference...
      import datetime

      # 'start' timestamp
      start = datetime.datetime.now()

      error_counter = 0

      # Report data (local scope)
      l_report_data = {}
      l_report_data['module']       = module
      l_report_data['time']         = None
      l_report_data['request_type'] = req_type
      l_report_data['data']         = []

      # Define how the attack modes should be handled
      l_attack_modes = {
         'generic': {'GET': Attack, 'POST': Attack},
         'post_dynamic_keys': {'GET': Attack, 'POST': AttackDynamicKeys}
      }
      
      # create new Attack instance for links and forms
      for i in data:

         #--- Create new attack handler
         f = l_attack_modes[opts['attack_modus']][req_type]
         attack_handler = f(i, attack_modules[module], req_type, opts)

         # start attack and save the results for reporting
         vuln_data = attack_handler.attack()

         # Set timestamp difference
         l_report_data['time'] = datetime.datetime.now() - start

         if vuln_data:
            # Append data
            l_report_data['data'] += [{'url': i, 'data': vuln_data}]

      self.g_report_data.append(l_report_data)


   def start_attack_modules(self, (options, args), xml_config, links, forms):
      """
      start chosen attack modules
      """

      self.options = options
      
      # Detect attack modus
      if not attack_modes.has_key(self.options.attack_modus):
          # Set default one
          self.options.attack_modus = attack_modes['generic']
      else:
          self.options.attack_modus = attack_modes[self.options.attack_modus]

      print "[+] attack modus: %s" % self.options.attack_modus

      # Split modules list, remove duplicates and 
      # send attacks with given modules
      modules = list(set(self.options.module.rsplit(',')))

      # Set several options
      opts['timeout'] = xml_config['target_req_timeout']
      opts['cookie_file'] = xml_config['acc_cookie']
      opts['payloads_max'] = xml_config['payloads_max']
      opts['attack_modus'] = self.options.attack_modus
      opts['payloads_file'] = xml_config['payloads_file']

      # ALL: default option
      if self.options.module == 'all':
         for i in attack_modules.keys():
            print START_ATTACK_MSG % attack_modules[i]
            if self.options.request_type == 'all':
               print GET_REQUEST_MSG
               self.send_attacks(i, links, 'GET')
               print POST_REQUEST_MSG
               self.send_attacks(i, forms, 'POST')
            else:
               if self.options.request_type.upper() == 'GET':
                  print GET_REQUEST_MSG
                  self.send_attacks(i, links, 'GET')

               elif self.options.request_type.upper() == 'POST':
                  print POST_REQUEST_MSG
                  self.send_attacks(i, forms, 'POST')
            print STOP_ATTACK_MSG % attack_modules[i]
         
      # Go through modules list
      else:
         for mod in modules:
            if mod in attack_modules.keys():
               self.options.module = mod
               print START_ATTACK_MSG % attack_modules[mod]
               if self.options.request_type == 'all':
                  print GET_REQUEST_MSG
                  self.send_attacks(mod, links, 'GET')
                  print POST_REQUEST_MSG
                  self.send_attacks(mod, forms, 'POST')
         
               else:
                  if self.options.request_type.upper() == 'GET':
                     print GET_REQUEST_MSG
                     self.send_attacks(mod, links, 'GET')

                  elif self.options.request_type.upper() == 'POST':
                     print POST_REQUEST_MSG
                     self.send_attacks(mod, forms, 'POST')
         
               print STOP_ATTACK_MSG % attack_modules[mod]

            else:
               print "[-] WARNING: unknown attack module: %s" % mod

      return self.g_report_data


   def report_results(self, (options, args), report_data):
      """
      choose report style to use
      """
     
      self.options = options

      # Check specified reporting style is supported
      if self.options.report in report_styles:
         print "[+] report: %s" % self.options.report
         report = ReportStyle()

         if self.options.report == 'text':
             report.stdout_report(report_data, UWSS_TXT_LOGFILE)
         elif self.options.report == 'xml':
             report.xml_report(report_data, UWSS_XML_LOGFILE)
         else:
             report.stdout_report(report_data)
      else:
         print '[-] FATAL: report unknown, aborting ...'
         sys.exit(0)


# EOF
