#    uwss - uber web security scanner
#    Copyright (C) 2009 noptrix & illuminatus
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# uError.py
#
# by noptrix & illuminatus
# www.noptrix.net


import os
import traceback


class Error(Exception):
    """ Global error handler class

    Attributes:
    -------------------------------
        classname - the class where the error occured
        message - explanation of the error


    Methods:
    -------------------------------
    __init__ (location,message)
        location = where did the error occured
        message = message to display

    print_error():
        Print specific Error message

    """


    def __init__(self, location, message):
        self.location = location
        self.message = message



class ConfigError(Error):
    """ Raise exceptions dealing with configuration issues """

    def print_error(self):
        message =  "--------------------------------------------------------\n" 
        message += "[---!---] CONFIG Error in %s:\n" % (self.location)
        message += "Details: %s" % (self.message,)
        message +=  "--------------------------------------------------------\n" 
        print message



class NetError(Error):
    """ Network (e.g. HTTP Requests) exceptions """

    # Table mapping response codes to messages; entries have the
    # form {code: (shortmessage, longmessage)}.
    responses = {
         100: ('Continue', 'Request received, please continue'),
         101: ('Switching Protocols','Switching to new protocol; obey Upgrade header'),
         200: ('OK', 'Request fulfilled, document follows'),
         201: ('Created', 'Document created, URL follows'),
         202: ('Accepted','Request accepted, processing continues off-line'),
         203: ('Non-Authoritative Information', 'Request fulfilled from cache'),
         204: ('No Content', 'Request fulfilled, nothing follows'),
         205: ('Reset Content', 'Clear input form for further input.'),
         206: ('Partial Content', 'Partial content follows.'),
         300: ('Multiple Choices','Object has several resources -- see URI list'),
         301: ('Moved Permanently', 'Object moved permanently -- see URI list'),
         302: ('Found', 'Object moved temporarily -- see URI list'),
         303: ('See Other', 'Object moved -- see Method and URL list'),
         304: ('Not Modified','Document has not changed since given time'),
         305: ('Use Proxy','You must use proxy specified in Location to access this resource.'),
         307: ('Temporary Redirect','Object moved temporarily -- see URI list'),
         400: ('Bad Request','Bad request syntax or unsupported method'),
         401: ('Unauthorized','No permission -- see authorization schemes'),
         402: ('Payment Required','No payment -- see charging schemes'),
         403: ('Forbidden','Request forbidden -- authorization will not help'),
         404: ('Not Found', 'Nothing matches the given URI'),
         405: ('Method Not Allowed','Specified method is invalid for this server.'),
         406: ('Not Acceptable', 'URI not available in preferred format.'),
         407: ('Proxy Authentication Required', 'You must authenticate with ''this proxy before proceeding.'),
         408: ('Request Timeout', 'Request timed out; try again later.'),
         409: ('Conflict', 'Request conflict.'),
         410: ('Gone','URI no longer exists and has been permanently removed.'),
         411: ('Length Required', 'Client must specify Content-Length.'),
         412: ('Precondition Failed', 'Precondition in headers is false.'),
         413: ('Request Entity Too Large', 'Entity is too large.'),
         414: ('Request-URI Too Long', 'URI is too long.'),
         415: ('Unsupported Media Type', 'Entity body in unsupported format.'),
         416: ('Requested Range Not Satisfiable','Cannot satisfy request range.'),
         417: ('Expectation Failed','Expect condition could not be satisfied.'),
         500: ('Internal Server Error', 'Server got itself in trouble'),
         501: ('Not Implemented','Server does not support this operation'),
         502: ('Bad Gateway', 'Invalid responses from another server/proxy.'),
         503: ('Service Unavailable','The server cannot process the request due to a high load'),
         504: ('Gateway Timeout','The gateway server did not receive a timely response'),
         505: ('HTTP Version Not Supported', 'Cannot fulfill request.')
    }


    def print_error(self):
        
        # Fatal HTTP error codes 
        message =  "--------------------------------------------------------\n" 
        message += "[---!---] NET Error in %s:\n" % (self.location)
        message += "Details: %s\n" % self.message
        message += "--------------------------------------------------------\n"
        print message



class AttackError(Error):
    """ Raise exceptions during attacks """

    def print_error(self):
        message =  "--------------------------------------------------------\n" 
        message += "[---!---] ATTACKS Error in %s:\n" % (self.location)
        message += "Details: %s" % (self.message,)
        message += "--------------------------------------------------------\n" 
        print message
