#    uwss - uber web security scanner
#    Copyright (C) 2009 noptrix & illuminatus
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# uXml.py
#
# by noptrix & illuminatus
# www.noptrix.net


import sys
import libxml2

# uwss imports
from uError import *
from uConfig import *
import __main__


## XML configuration class
class xmlParser:
   """ Parse XML file

       Methods:
       -------------------------------
       __init__ (filename,opts):
         File name to be parsed + options

       xml_check_validity():
         Check if configuration (XML) file is valid

       xml_read_config():
         Parse XML file and return dictionary containing config options

   """

   ## The constructor
   def __init__(self, file_name, opts):
      self.opts = opts
      #----------------------------------------------------------------
      if opts["DEBUG"]:
         print '[DEBUG] xmlParser: Created object (file_name = %s)' \
                  % file_name
      #----------------------------------------------------------------
      self.xml_file = file_name


   def xml_check_validity(self):
      """ Checks if the XML document refered by 'self.xml_file' is valid
          @return 1 for valid XML data, 0 for invalid
      """
      if self.xml_file:

         # Create parser context
         xml_ctxt = libxml2.createFileParserCtxt(self.xml_file)
         
         # Switch the parser to validation mode
         xml_ctxt.validate(1)

         # Parse the XML document
         xml_ctxt.parseDocument()

         # Get the document tree
         doc = xml_ctxt.doc()

         # Check validation result
         valid = xml_ctxt.isValid()

         # Free doc
         doc.freeDoc()

         return valid


   def xml_read_config(self):
      """ Reads configuration vars by XPATH from XML file
          @return dict Dictionary containing the config vars
      """
      try:
         # Parse file
         #--------------------------------------------------------
         if opts["DEBUG"]:
            print '[DEBUG] -- xml_read_config: parsing file...'

         if opts["VERBOSE"]:
            print '[*] CONFIG: Reading configuration file'
         #--------------------------------------------------------
         doc = libxml2.parseFile(self.xml_file)
         
         # Create XPATH context
         xpath_ctxt = doc.xpathNewContext()

         # ---- Read information from XML document ----
         conf = {}                                           # global
         conf_target = {}                                    # general
         conf_crawler= {}                                    # crawler
         conf_account = {}                                   # account
         conf_payloads = {}                                  # payloads
            
         # General information
         conf_target['target_url']          = '//target/url'
         conf_target['target_req_timeout']  = '//target/timeout'

         # Crawler information
         conf_crawler['crawler_user_agent']      = '//crawler/user-agent'
         conf_crawler['crawler_deep_level']      = '//crawler/depth-level'
         conf_crawler['crawler_verbosity']       = '//crawler/verbosity'
         conf_crawler['crawler_exclude_urls']    = '//crawler/excluded-urls//url'
         conf_crawler['crawler_exclude_post_params'] = \
                                                 '//crawler/excluded-post-params//entry'
         conf_crawler['crawler_website_data']    = '//crawler/website-data'

         #Login account information
         conf_account['acc_url']            = '//account/url'
         conf_account['acc_field_user']     = '//account/field_user'
         conf_account['acc_field_passwd']   = '//account/field_passwd'
         conf_account['acc_user']           = '//account/user'
         conf_account['acc_passwd']         = '//account/passwd'
         conf_account['acc_cookie']         = '//account/cookie-data'

         # Payloads information
         conf_payloads['payloads_file']     = '//payloads/@file'
         conf_payloads['payloads_max']      = '//payloads/@max'


         #--------------------------------------------------------
         if opts["VERBOSE"]:
            print '[*] CONFIG: Fetch configuration options'
         #--------------------------------------------------------
         
         # Fetch general settings
         for x in conf_target.keys():
            #--------------------------------------------------------
            if opts["DEBUG"]:
               print '[DEBUG] -- xml_read_config: conf[\'%s\'] \t--> %s' \
                      % (x,xpath_ctxt.xpathEval(conf_target[x])[0].content)
            #--------------------------------------------------------

            conf[x] = xpath_ctxt.xpathEval(conf_target[x])[0].content

         
         # Fetch crawler stuff
         for x in conf_crawler.keys():
            # Create list of excluded URLs
            if x == "crawler_exclude_urls":
               exclude_list = []
               res = xpath_ctxt.xpathEval(conf_crawler[x])

               # Append to exclude_list
               for i in res:
                  exclude_list.append(i.content)
               
               conf[x] = exclude_list

            # Create list of excluded POST parameters
            elif x == "crawler_exclude_post_params":

               # Our list is actually a dictionary since indexing
               # is much easier
               exclude_list = {}
               res = xpath_ctxt.xpathEval(conf_crawler[x])

               # Append to exclude_list
               for i in res:
                  entry = libxml2.parseDoc(str(i))
                  url = entry.xpathEval('/entry/@url')[0].content

                  # Get the parameters list
                  # ...but first of all get rid of spaces
                  param_list = entry.xpathEval('/entry/param-list')[0].content
                  param_list = param_list.replace(" ","")
                  param_list = param_list.split(',')

                  # Create list entry
                  exclude_list[url] = param_list
               
               conf[x] = exclude_list


            # Add to global conf as usually
            else:
               conf[x] = xpath_ctxt.xpathEval(conf_crawler[x])[0].content


         # Fetch account stuff
         for x in conf_account.keys():
            conf[x] = xpath_ctxt.xpathEval(conf_account[x])[0].content


         # Fetch payloads stuff
         for x in conf_payloads.keys():
            conf[x] = xpath_ctxt.xpathEval(conf_payloads[x])[0].content
         
         # Return results
         return conf

      except Exception, e:
         raise ConfigError('xmlParser',e)

# EOF
