# lswww v2.1.5 - A web spider library
# Copyright (C) 2006 Nicolas Surribas
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# NOTE: Minor changes and adapted to uwss by illuminatus
#

import sys, re, socket, getopt, os
import HTMLParser, urllib, urllib2, cookielib

# uwss imports
from uhttp.uHttp import *

try:
    import tidy

except ImportError:
	#print "lswww will be far less effective without tidy"
	#print "please install libtidy ( http://tidy.sourceforge.net/ ),"
	#print "ctypes ( http://starship.python.net/crew/theller/ctypes/ )"
	#print "and uTidylib ( http://utidylib.berlios.de/ )"
	tidyhere=0
else:
	tidyhere=1

BeautifulSouphere = 0

class lswww:
    """ Some cool web spider library """
    root = ""
    server = ""
    tobrowse = []
    browsed = []
    proxy = {}
    excluded = []

    # Exclude POST parameters from being replaced by "%CHANGEME%"
    excluded_post_params = {}
    
    forms = []
    uploads = []
    allowed = ['php', 'html', 'htm', 'xml', 'xhtml', 'xht', 'xhtm',
               'asp', 'aspx', 'php3', 'php4', 'php5', 'txt', 'shtm',
               'shtml', 'phtm', 'phtml', 'jhtml', 'pl', 'jsp', 'cfm', 'cfml']
    verbose = 0
    cookie = ""
    auth_basic = []
    bad_params = []
    timeout = 6

    http_headers = {}

    # Default settings
    max_deep = 3


    def __init__(self, rooturl, opts):
        """ The constructor """
        root = rooturl

        # Global options
        self.opts = opts
        
        # Add trailing slash if needed
        #if root[-1] != "/":
        #    root += "/"

        # Check if link is valid
        if(self.checklink(root)):
            print "Invalid link argument"
            sys.exit(0)

        # Build hostname uppon given URL
        server = (root.split("://")[1]).split("/")[0]

        # Set root
        self.root = root.split("/")[0] + '//' + server
        self.server = server

        # The rooturl will be our first URL to scan for
        self.tobrowse.append((0,root))


    def setTimeOut(self, timeout=6):
        """Set the timeout in seconds to wait for a page"""
        self.timeout = timeout


    def setMaxDeep(self, deep):
        """ Set maximal link deep """
        self.max_deep = deep


    def setProxy(self, proxy={}):
        """Set proxy preferences"""
        self.proxy = proxy


    def setHeaders(self,headers):
        """ Set HTTP headers to send within requests """
        self.http_headers = headers


    def addStartURL(self, url):
        if(self.checklink(url)):
            print "Invalid link argument:", url
            sys.exit(0)
        if(self.inzone(url) == 0):
            self.tobrowse.append(url)


    def addExcludedURL(self, url):
        """Add an url to the list of forbidden urls"""
        self.excluded.append(url)


    def setCookieFile(self, cookie):
        """Set the file to read the cookie from"""
        self.cookie = cookie


    def setAuthCredentials(self, auth_basic):
        self.auth_basic = auth_basic


    def addBadParam(self, bad_param):
        self.bad_params.append(bad_param)


    def browse(self, url, url_deep):
        """ Extract urls from a webpage and add them to the list of urls
            to browse if they aren't in the exclusion list
        """
        # We don't need destination anchors
        current = url.split("#")[0]

        # Url without query string
        current = current.split("?")[0]

        # Get the dirname of the file
        currentdir = "/".join(current.split("/")[:-1]) + "/"

        try:
            u = self.g_req.send_GET_request(url)

        # BadStatusLine can happen when no HTTP status code is given or
        # when a connexion is suddenly closed
        except urllib2.httplib.BadStatusLine:
            print "Error reading response"
            return 0

        except IOError, e:
            print "\n" + url + ":", e

            # Exclue URL since its not accessible
            self.excluded.append(url)
            return 0

        except NetError,e:
            self.excluded.append(url)
            return 0
        
        # Find out what kind of proto: HTTP, HTTPS etc.
        proto = url.split("://")[0]

        if proto == "http" or proto == "https":
            # Check the content-type fiest
            if not u.info().get("Content-Type"):
                # Sometimes there's no content-type... so we rely on the document extension
                if (current.split(".")[-1] not in self.allowed) and current[-1] != "/":
                    return 1

            elif u.info().get("Content-Type").find("text") == -1:
                return 1

        # Manage redirections
        if u.headers.dict.has_key("location"):
            redir = self.correctlink(u.headers.dict["location"], current, currentdir, proto)
            if redir != None:
                if(self.inzone(redir) == 0):
                    # Is the document already visited of forbidden ?
                    if (redir in self.browsed) or (redir in self.tobrowse) or self.isExcluded(redir):
                        pass
                    else:
                        # No -> Will browse it soon
                        self.tobrowse.append(redir)
        try:
            htmlSource = unicode(u.read(), errors="replace")
        except socket.timeout:
            htmlSource = ""

        p = linkParser()

        # Set list of excluded POST params
        p.excluded_post_params = self.excluded_post_params
        
        try:
            p.feed(htmlSource)
        except HTMLParser.HTMLParseError, err:
            if tidyhere == 1:
                options = dict(output_xhtml=1, add_xml_decl=1, indent=1, tidy_mark=0)
                htmlSource = str(tidy.parseString(htmlSource,  ** options))
                try:
                    p.reset()
                    p.feed(htmlSource)
                except HTMLParser.HTMLParseError, err:
                    pass
            elif BeautifulSouphere == 1:
                htmlSource = BeautifulSoup.BeautifulSoup(htmlSource).prettify()
                try:
                    p.reset()
                    p.feed(htmlSource)
                except HTMLParser.HTMLParseError, err:
                    pass
            # last chance
            else:
                p.liens = re.findall('href="(.*?)"', htmlSource)

        for lien in p.uploads:
            self.uploads.append(self.correctlink(lien, current, currentdir, proto))

        for lien in p.liens:
            lien = self.correctlink(lien, current, currentdir, proto)

            if lien != None:
                if(self.inzone(lien) == 0):
                    # Is the document already visited or forbidden ?
                    if (lien in self.browsed) or (lien in self.tobrowse) or self.isExcluded(lien):
                        pass
                    else:
                        # No -> Will browse it soon
                        self.tobrowse.append((url_deep + 1,lien))

        for form in p.forms:
            action = self.correctlink(form[0], current, currentdir, proto)
            if action == None: action = current
            form = (action, form[1], url)
            if form not in self.forms: self.forms.append(form)
            
        # We automaticaly exclude 404 urls
        if u.code == 404:
            self.excluded.append(url)
            return 0
        return 1


    def correctlink(self, lien, current, currentdir, proto):
        """Transform relatives urls in absolutes ones"""

        # No leading or trailing whitespaces
        lien = lien.strip()

        # bad protocols
        if lien.find("telnet:", 0) == 0 or lien.find("ftp:", 0) == 0 or lien.find("mailto:", 0) == 0 or \
            lien.find("javascript:", 0) == 0 or lien.find("news:", 0) == 0 or lien.find("file:", 0) == 0 or \
            lien.find("gopher:", 0) == 0 or lien.find("irc:", 0) == 0 or lien == "":
            return None

        # Good protocols or relatives links
        else:
            # full url, nothing to do :)
            if (lien.find("http://", 0) == 0) or (lien.find("https://", 0) == 0):
                pass
            else:
                # root-url related link
                if(lien[0] == '/'):
                    lien = proto + "://" + self.server + lien
                else:
                    # same page + query string
                    if(lien[0] == '?'):
                        lien = current + lien

                    # current directory related link
                    else:
                        lien = currentdir + lien

            # No destination anchor
            if lien.find("#") != -1:
                lien = lien.split("#")[0]

            # reorganize parameters in alphabetical order
            if lien.find("?") != -1:
                args = lien.split("?")[1]
                if args.find("&") != -1:
                    args = args.split("&")
                    args.sort()
                    args = [i for i in args if i != "" and i.find("=") >= 0]
                    for i in self.bad_params:
                        for j in args:
                            if j.startswith(i + "="): args.remove(j)
                    args = "&".join(args)

                # a hack for auto-generated Apache directory index
                if args in ["C=D;O=A", "C=D;O=D", "C=M;O=A", "C=M;O=D", "C=N;O=A", "C=N;O=D", "C=S;O=A", "C=S;O=D"]:
                    lien = lien.split("?")[0]
                else:
                    lien = lien.split("?")[0] + "?" + args

            # Remove the trailing '?' if its presence doesn't make sense
            if lien[-1:] == "?":
                lien = lien[:-1]

            # remove useless slashes
            if lien.find("?") != -1:
                file = lien.split("?")[0]
                file = re.sub("[^:]//+", "/", file)
                lien = file + "?" + lien.split("?")[1]

            # links going to a parrent directory (..)
            while re.search("/([~:!,;a-zA-Z0-9\.\-+_]+)/\.\./", lien) != None:
                lien = re.sub("/([~:!,;a-zA-Z0-9\.\-+_]+)/\.\./", "/", lien)

            lien = re.sub("/\./", "/", lien)

            # Everything is good here
            return lien


    def checklink(self, url):
        """Verify the protocol"""
        if (url.find("http://", 0) == 0) or (url.find("https://", 0) == 0):
            return 0
        else:
            return 1


    def inzone(self, url):
        """Make sure the url is under the root url"""
        if(url.find(self.root, 0) == 0):
            return 0
        else:
            return 1


    def isExcluded(self, url):
        """Return True if the url is not allowed to be scan"""
        match = False
        for regexp in self.excluded:
            if self.reWildcard(regexp, url):
                match = True
        return match


    def reWildcard(self, regexp, string):
        """Wildcard-based regular expression system"""
        regexp = re.sub("\*+", "*", regexp)
        match = True
        if regexp.count("*") == 0:
            if regexp == string:
                return True
            else:
                return False
        blocks = regexp.split("*")
        start = ""
        end = ""
        if not regexp.startswith("*"):
            start = blocks[0]
        if not regexp.endswith("*"):
            end = blocks[-1]
        if start != "":
            if string.startswith(start):
                blocks = blocks[1:]
            else:
                return False
        if end != "":
            if string.endswith(end):
                blocks = blocks[:-1]
            else:
                return False
        blocks = [block for block in blocks if block != ""]
        if blocks == []:
            return match
        for block in blocks:
            i = string.find(block)
            if i == -1: return False
            string = string[i + len(block):]
        return match

    
    def go(self):
        """ Starts collecting URL's from rooturl and preprocesses them"""

        # Set global request handler
        self.g_req = httpRequest(self.opts)
       
        # Set http headers
        self.g_req.set_headers(self.http_headers)
   
        # while url list isn't empty, continue browsing
        # if the user stop the scan with Ctrl+C, give him all found urls
        try:
            while len(self.tobrowse) > 0:
                
               # Fetch last inserted argument from list
                cur_element = self.tobrowse.pop(0)
                deep = cur_element[0]                               # URL deep
                lien = cur_element[1]                               # URL

                # Browse link otherwise mark it as 'browsed'
                if (lien not in self.browsed):
                    # No more than 'max_deep'
                    if deep + 1 <= self.max_deep:
                        if self.browse(lien,deep):
                            self.browsed.append(lien)
                    else:
                        self.browsed.append(lien)

                # DEBUG #
                # to_browse = self.tobrowse

                # Verbosity options
                if self.verbose == 1:
                    sys.stderr.write('.')

                elif self.verbose == 2:
                    sys.stderr.write('URL Deep: ' + str(deep) + '\t' \
                    + 'ToBrowse: ' + str(len(self.tobrowse)) + '\t' \
                    + 'Browsed: ' + str(len(self.browsed)) + '\t' \
                    + '\t' + lien + "\n")
            if self.verbose == 1:
               print "\n",

        except KeyboardInterrupt:
            pass


    def verbosity(self, vb):
        """Set verbosity level"""
        self.verbose = vb


    def printLinks(self):
        """Print found URLs on standard output"""
        self.browsed.sort()
        sys.stderr.write("\n+ URLs :\n")
        for lien in self.browsed:
            print lien


    def printForms(self):
        """Print found forms on standard output"""
        if self.forms != []:
            sys.stderr.write("\n+ Forms Info :\n")
            for form in self.forms:
                print "From:", form[2]
                print "To:", form[0]
                for k, v in form[1].items():
                    print "\t" + k, ":", v
                print


    def printUploads(self):
        """Print urls accepting uploads"""
        if self.uploads != []:
            sys.stderr.write("\n+ Upload Scripts :\n")
            for up in self.uploads:
                print up


    def getLinks(self):
        self.browsed.sort()
        return self.browsed


    def getForms(self):
        return self.forms


    def getUploads(self):
        self.uploads.sort()
        return self.uploads

class linkParser(HTMLParser.HTMLParser):
    """Extract urls in 'a' href HTML tags"""
    def __init__(self):
        HTMLParser.HTMLParser.__init__(self)
        self.liens = []
        self.forms = []
        self.form_values = {}
        self.inform = 0
        self.current_form_url = ""
        self.uploads = []
        self.current_form_method = "get"
        self.excluded_post_params = {}

    def handle_starttag(self, tag, attrs):
        tmpdict = {}
        val = None
        for k, v in dict(attrs).items():
            tmpdict[k.lower()] = v


        # Normal a hrefs
        if tag.lower() == 'a':
            if "href" in tmpdict.keys():
                self.liens.append(tmpdict['href'])

        # Forms
        if tag.lower() == 'form':
            self.inform = 1
            self.form_values = {}

            if "action" in tmpdict.keys():
                self.liens.append(tmpdict['action'])
                self.current_form_url = tmpdict['action']
        
            # Find out which kind of method the form is using
            # Forms use GET method by default
            self.current_form_method = "get"

            if "method" in tmpdict.keys():
                if tmpdict["method"].lower() == "post":
                    self.current_form_method = "post"

        # Input areas
        if tag.lower() == 'input':

            # Is this a form?
            if self.inform == 1:
                if "type" not in tmpdict.keys():
                    tmpdict["type"] = "text"

                # Fetch input variables name
                if "name" in tmpdict.keys():
                    if tmpdict['type'].lower() in ['text', 'password', 'radio', 'checkbox', 'hidden', 'submit', 'search']:
                        # use default value if present or set it to 'on'
                        if "value" in tmpdict.keys():
                
                            # If current tag is excluded don't replace its value by "%CHANGEME%"
                            if self.excluded_post_params.has_key(self.current_form_url):
                                if tmpdict['name'] in self.excluded_post_params[self.current_form_url]:
                                    if tmpdict["value"] != "":
                                        val = tmpdict["value"]
                                    else:
                                        val = "%CHANGEME%"
                            else: 
                                    val = "%CHANGEME%"
                        else:
                            val = "%CHANGEME%"

                        self.form_values.update(dict([(tmpdict['name'], val)]))

                    if tmpdict['type'].lower() == "file":
                        self.uploads.append(self.current_form_url)

        if tag.lower() in ["textarea", "select"]:
            if self.inform == 1:
                if "name" in tmpdict.keys():
                    # If current tag is excluded don't replace its value by "%CHANGEME%"
                    if self.excluded_post_params.has_key(self.current_form_url):
                        if tmpdict['name'] in self.excluded_post_params[self.current_form_url]:
                            if tmpdict["value"] != "":
                                val = tmpdict["value"]
                            else:
                                val = "%CHANGEME%"
                    else:
                            val = "%CHANGEME%"
                    self.form_values.update(dict([(tmpdict['name'], val)]))

        if tag.lower() in ["frame", "iframe"]:
            if "src" in tmpdict.keys():
                self.liens.append(tmpdict['src'])

    def handle_endtag(self, tag):
        if tag.lower() == 'form':
            self.inform = 0
            if self.current_form_method == "post":
                self.forms.append((self.current_form_url, self.form_values))
            else:
                l = ["=".join([k, v]) for k, v in self.form_values.items()]
                l.sort()
                self.liens.append(self.current_form_url.split("?")[0] + "?" + "&".join(l))
