#    uwss - uber web security scanner
#    Copyright (C) 2009 noptrix & illuminatus
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# uCrawler.py
#
# by noptrix && illuminatus
# www.noptrix.net


import pickle
import re

# uwss imports
from lswww import *
from uconfig.uConfig import *


class crawler(lswww):
   """ Crawl some URL and provide functions to fetch links,forms etc.


       Attributes (also see lswww.py):
       -------------------------------
       root:
         Start url

       tobrowse[] (inherited):
         List of URLs that have to be browsed

       browsed[] (inherited):
         List of browsed URLs)

       forms[] (inherited):
         List of collected POST forms data

       exclude_post_params[] (inherited):
         List of excluded POST parameters


       Methods:
       -------------------------------
       __init__ (starturl,options):
         Excepts some start url and options, e.g. VERBOSE or DEBUG

       preprocess_links(): 
         Replace values (in GET variables) by %CHANGEME%

       get_links():
         Return list of browsed URLs

       get_forms():
         Return list of FORMS data

       save_data(filename):
         Save collected data (using pickle) to file

       read_data(filename):
         Read data (using pickle) from file

       set_excluded_post_param_list(list):
         Set excluded POST parameters list    
   """

   def __init__(self, starturl, opts):
      self.opts = opts
      #----------------------------------------------------------------
      if opts["DEBUG"]:
         print '[DEBUG] crawler: Created object ( starturl = %s)' \
                  % starturl
      #----------------------------------------------------------------
      lswww.__init__(self, starturl, opts)


   def preprocess_links(self):
      """ Replace GET variables content in URLS by '%CHANGEME%' """

      new_browsed = []
      for link in self.browsed:
         p = re.compile('\?.*$')                   # regexp pattern
         m = p.search(link)

         ###### PHP GET variables
         # Example: http://foobar.com/?param1=value1&param2=value2..
         if m:                                     # regexp matches!
            item = m.group()

            # Parse URL
            from urlparse import urlparse
            parser = urlparse(link)
            
            p_root = parser.scheme + '://' + parser.netloc + parser.path

            # Split URL string by "&" --> ["param1=value1","param2=value2",...]
            items = item.split("&")

            for i in items:
               # Replace GET variables value by "%CHANGEME%"
               l_items = items[:]
               val = i.split('=')
               val[-1] = '%CHANGEME%'
               b = val[0] + '=' + val[-1]

               l_items[l_items.index(i)] = b

               # Concatenate variables
               k = p_root + "&".join(l_items)

               # Generate new URL
               new_browsed.append(k)


         ###### Clean URLs
         # Example: http://foobar.com/parm1/value1/...
         else:
             # Links list
             urls_list = []

             # Get URL basic
             p_root = re.compile('(http.*\/\/.*?\/)')
             m_root = p_root.search(link)

             # Remove the base from URL and split arguments
             if m_root:
                rest = link.split(m_root.group())[1].split('/')

                # Reverse order of our list
                rest.reverse()

                for r in rest:
                    # Create local list
                    l = rest[:]

                    # Replace current item by %CHANGEME%
                    # ... and reverse it to its original order
                    l[l.index(r)] = "%CHANGEME%"
                    l.reverse()

                    # Create new URL string
                    new_url = m_root.group() + "/".join(l)
                    new_browsed.append(new_url)

      # Update 'self.browsed'
      for i in new_browsed:
         self.browsed.append(i)
        
   
   def get_links(self):
      """  Get collected links """
      self.browsed.sort()
      return self.browsed


   def save_data(self, filename):
      """ Save collected data to file using 'pickle' """

      #----------------------------------------------------------------
      if opts["VERBOSE"] or opts["DEBUG"]:
         print '[*] CRAWLER: save collected data to %s' % filename
      #----------------------------------------------------------------

      # Create file handler for file
      fd = open(filename, 'wb')

      # Get data to save (create unique list)
      _urls = list(set(self.get_links()))               # unique list
      _forms = self.get_forms()

      # Dump data to file: URLs, Forms
      pickle.dump(_urls, fd)
      pickle.dump(_forms, fd)

      # Close file handler
      fd.close()


   def read_data(self, filename):
      """ Read data from file using 'pickle' """

      #----------------------------------------------------------------
      if opts["VERBOSE"] or opts["DEBUG"]:
         print '[*] CRAWLER: read collected data from %s' % filename
      #----------------------------------------------------------------

      # Open file
      fd = open(filename, 'rb')

      # Load data to inherited variables
      self.browsed = pickle.load(fd)
      self.forms = pickle.load(fd)

      # Close file
      fd.close()


   def set_excluded_post_param_list(self, excluded_list):
      """ Set excluded POST parameters list """
      self.excluded_post_params = excluded_list


   def get_forms(self):
      return self.forms

# EOF
