#    uwss - uber web security scanner
#    Copyright (C) 2009 noptrix & illuminatus
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# uSQL.py
#
# by noptrix & illuminatus
# www.noptrix.net


import sqlite3, os

# uwss imports
from uconfig.uError import *
from uconfig.uConfig import *


class attacksDB:
    """ Provide SQLite functionalities to the attacks DB """

    # SQLite file handler
    __conn = None


    def __init__(self, filename, opts):
        self.opts = opts
        #----------------------------------------------------------------
        if opts["DEBUG"]:
            print '[DEBUG] attacksDB: Created object (filename = %s)' \
                  % filename
        #----------------------------------------------------------------
        self.filename = filename


    # Destructor
    def __del__(self):
        self.disconnect()


    def connect(self):
        """ Opens a connection to the SQLite database file """
        #----------------------------------------------------------------
        if opts["DEBUG"]:
            print '[DEBUG] -- connect: Connecting to SQLite3 database \'%s\'' \
                  % self.filename

        if opts["VERBOSE"]:
            print '[*] DB: Connected to DB'
        #----------------------------------------------------------------

        self.__conn = sqlite3.connect(self.filename)


    def disconnect(self):
        """ Closes the database connection """
        self.__conn.close()


    def show_attacks(self):
        """ Show all available attacks in the DB file """

        # Create cursor
        self.c = self.__conn.cursor()

        try:
            # Send query
            self.c.execute('select * from attacks_data')
        except Exception, e:
               raise NetError('attackDB/show_attacks()',e)


        # Iterate through rows
        for row in self.c:
            print row

        # Close self.cursor
        self.c.close()


    def show_attacks_by_type(self, module, limit=5):
        """ Fetch available attacks in the DB by module name
            @return List of tuples
        """

        self.results = []

        self.c = self.__conn.cursor()

        try:
            # Build query and send it
            #----------------------------------------------------------------
            if opts["DEBUG"]:
                print '[DEBUG] -- show_attacks_by_type: module = %s' % module

            if opts["VERBOSE"]:
                print '[*] DB: Fetching data from DB...'
            #----------------------------------------------------------------

            self.m = (module, limit)
            self.c.execute('SELECT ID,type,data,success_data FROM attacks_data \
                            WHERE type=? LIMIT ?',self.m)

        except Exception, e:
            raise NetError('attacksDB/show_attacks_by_type()',e)

        # Return data
        for row in self.c:
            #----------------------------------------------------------------
            if opts["DEBUG"]:
                print '[DEBUG] -- show_attacks_by_type: row = ', row
            #----------------------------------------------------------------
            self.results.append(row)

        # Close cursor
        self.c.close()

        return self.results


    def show_attacks_by_id(self, id):
        """ Fetch attack by ID
            @return If found return list of row(s)
        """
        self.results = []

        self.c = self.__conn.cursor()
        self.id = (id,)

        try:
            self.c.execute('SELECT ID,type,data,success_data FROM attacks_data\
                  WHERE ID=?',self.id)
        except Exception,e:
            raise NetError('attacksDB/show_attacks_by_id()',e)

        # Return data
        for row in self.c:
            self.results.append(row)

        # Close cursor
        self.c.close()

        return self.results


    def insert_attack(self):
        """ Insert new attack """
        pass


    def delete_attack(self):
        """ Delete attack """
        pass


# EOF
