#    uwss - uber web security scanner
#    Copyright (C) 2009 noptrix & illuminatus
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# uHttp.py
#
# by noptrix & illuminatus
# www.noptrix.net


import urllib
import urllib2
import uCookie
import socket

# uwss imports
from uconfig.uError import *
from uconfig.uConfig import *


class httpRequest():

   # Private variables
   __http_headers = {}                                         # HTTP headers
   __http_conn = None                                          # HTTP handler
   __opener = None                                             # global opener

   __default_timeout = 6                                       # default timeout
   cookies = None                                              # Cookies class



   def __init__(self, opts):
      self.opts = opts
      """ The constructor """
      #----------------------------------------------------------------
      if opts["DEBUG"]:
         print '[DEBUG] httpRequest: Created object'
      #----------------------------------------------------------------

      # Set default handler
      self.__opener = urllib2.build_opener(urllib2.HTTPHandler)

      # ... add other ones
      self.__opener.add_handler(urllib2.HTTPSHandler())

      # Create cookies class
      self.cookies = uCookie.cooky(opts)

      # Set cookie file if available
      if opts.has_key('cookie_file'):
         self.cookies.load(opts['cookie_file'])
         self.set_cookie_handler()
         
      # Set default socket timeout
      socket.setdefaulttimeout(float(opts['timeout']))


   def set_opener(self, opener):
      """ Set opener director """
      self.__opener = opener


   def get_opener(self):
       """ Return opener director """
       return self.__opener


   def send_GET_request(self, url):
      """ Sends a GET request to a give URL

          Arguments
            url: Where to send the request
      """

      try:
         # Send request
         #----------------------------------------------------------------
         if opts["DEBUG"]:
            print '[DEBUG] -- send_GET_request: url = %s' % url

         if opts["VERBOSE"]:
            print '[*] NET: Send GET request to %s' % url
         #----------------------------------------------------------------

         req = urllib2.Request(url, None, self.__http_headers)
         data = self.__opener.open(req)

      except Exception,e:
         raise NetError('httpRequest/send_GET_request()', e)


      #----------------------------------------------------------------
      if opts["DEBUG"]:
         print '[DEBUG] -- send_GET_request: data.info = %s' \
               % self.data.info()
      #----------------------------------------------------------------
      return data


   def send_POST_request(self, url, values):
      """ Sends a POST request to a given URL

          Arguments:
            values: Dictionary containing values to send
      """

      #----------------------------------------------------------------
      if opts["DEBUG"]:
         print '[DEBUG] -- send_POST_request: url = %s' % url

      if opts["VERBOSE"]:
         print '[*] Send POST request to %s' % url
      #----------------------------------------------------------------

      # Check if values is dict
      if not (type(values) is dict):
         print 'No dict specified.'
         return

      # Uencode data
      data = urllib.urlencode(values)

      try:
         # Send request
         #----------------------------------------------------------------
         if opts["DEBUG"]:
            print '[DEBUG] -- send_POST_request: values = ',values
         #----------------------------------------------------------------

         req = urllib2.Request(url, data, self.__http_headers)
         ret = self.__opener.open(req)
         
      except Exception,e:
         raise NetError('httpRequest/send_POST_request()', e)

      #----------------------------------------------------------------
      if opts["DEBUG"]:
         print '[DEBUG] -- send_POST_request: data.info = %s' \
                % ret.info()
      #----------------------------------------------------------------
      return ret


   def sendRequest(self, method, url):
      """ Get response from a http request

          Attributes:
            method: GET or POST
            host: The host where to connect to
            url: The URL to send to server
            return Tuple of http return code + reason
      """
      # Send requests to host
      if method.upper() == 'GET':
         self.__http_conn.request("GET", url)                 # GET request

      elif method.upper() == 'POST':
         print "not implemented yet"                          # POST request

      else:
         print "unknown method"

      # Get HTTP headers
      rl = __http_conn.getresponse()

      # Return headers
      return (rl.status, rl.reason)


   def set_headers(self, values):
      """ Set HTTP headers used for the requests

          Attributes
            values: A dict containing values
      """
      if type(values) is dict:
         for v in values.keys():
            #-----------------------------------------------------------
            if opts["DEBUG"]:
               print '[DEBUG] -- set_headers: header[\'%s\'] = %s' \
                        % (self.__http_headers[v], values[v])
            #-----------------------------------------------------------
            self.__http_headers[v] = values[v]


   def get_headers(self):
      """ Get HTTP headers """
      return self.__http_headers


   def set_basicauth_handler(self, url, user, password):
      """ Set authentification credentials for Basic/Digest etc. auth.

          Attributes
            url: URL
            user: User needed for auth
            password: User's password
      """

      #----------------------------------------------------------------
      if opts["DEBUG"]:
         print '[DEBUG] -- set_basicauth_handler: url = %s\tuser = %s' \
                  '\tpassword = %s' % (url, user, password)

      if opts["VERBOSE"]:
         print '[*] NET: Set authentification credentials'
      #----------------------------------------------------------------

      # Create some password manager
      self.passmngr = urllib2.HTTPPasswordMgrWithDefaultRealm()
      self.passmngr.add_password(None, url, user, password)

      # Create authentificaton handler
      self.auth = urllib2.HTTPBasicAuthHandler(self.passmngr)
      self.auth.add_password(None, url, user, password)

      # Install the AuthHandler
      self.__opener.add_handler(self.auth)


   def set_cookie_handler(self):
      """ Set opener handler for cookies """
      self.__opener.add_handler(urllib2.HTTPCookieProcessor(self.cookies.cj))


   def install_handler(self):
      """ Install opener handler GLOBALLY """
      urllib2.install_opener(self.__opener)


# EOF
