#    uwss - uber web security scanner
#    Copyright (C) 2009 noptrix & illuminatus
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# uReport.py
#
# by noptrix & illuminatus
# www.noptrix.net


import libxml2, os
from datetime import date

# uwss imports
from uconfig.uConfig import *


class ReportStyle():
   """
   report style
   """

   def __init__(self):
      pass


   def data_wrapper(self, report_data):
      """
      Report data wrapper
      """

      return_data = []
      
      # If report data empty then do nothing
      if not report_data:
         print "[-] warning: report data empty. aborting..."
         return

      for i in report_data:
         if i:
            import pprint
            pprint.pprint(i)
            for j in i:
                return_data.append(j)

      return return_data

     
   def xml_report(self, report_data, filename=None):
      """
      output results to xml file
      """

      output = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
      output += "<uwss-stats date=\"" + str(date.today()) + "\">\n"
  
      # Iterate through report_data and add to output
      for i in report_data:

         # Calculate total amount of found data
         c = 0
         for l in i['data']: c += len(l['data'])

         output += "\t<attack request_type=\"" + i['request_type'] + "\" entries=\"" \
            + str(c) + "\">\n"                             # attack type
       
         output += "\t\t<targets>\n"

         # Loop trough targets
         for j in i['data']:

            # Check for request types
            if i['request_type'] == "GET":
               output += "\t\t\t<attack attack_type=\"" + i['module'] +"\">\n"

               for m in j['data']:
                    output +="\t\t\t\t<entry url=\"" + m + "\" />\n"

               output += "\t\t\t</attack>\n"

            elif i['request_type'] == "POST":
               output += "\t\t\t<attack attack_type=\"" + i['module'] +"\">\n"
               
               # Print POST variables
               for p in j['data']:
                  k = p.keys()[0]
                  output += "\t\t\t\t<post-data var=\"" + k + "\">"
                  output +=  p[k] + "</post-data>\n"

               output += "\t\t\t</attack>\n"


         output += "\t\t</targets>\n"
         output += "\t</attack>\n" 

      output += "</uwss-stats>"

      # Write results to file
      if filename:
         fd = open(filename,"w")
         fd.write(output)
         fd.close()
      else:
         print output


   def stdout_report(self, report_data, filename=None):
      """
      output results to stdout
      """
      _spaces = 10
      _lines = 75

      # Total elapsed time
      total_time = 0
      
      output = ''.join([s.ljust(20) for s in \
                     ('attack', 'hits', 'type','duration')]) + "\n"
      output += ('-' * _lines) + '\n'

      #----- General data
      for i in report_data:

         # Calculate total amount of found data
         c = 0
         for l in i['data']: c += len(l['data'])

         # Duration
         l_time = i['time'].seconds
         if l_time <= 0:
             l_time = '-'
         else:
             l_time = str(l_time) + 's'
             
         line = [i['module'], str(c), i['request_type'], l_time]
         output += ''.join([str(s).ljust(20) for s in line]) + "\n"
      
      output +=  ('-' * _lines) + '\n\n\n'


      #----- Attack details
      output += "attack details: \n" + "-" * _lines + "\n"
      
      # Print details
      for i in report_data:

         # Empty data ...
         if not i['data']:
             continue
             
         output += "[+] vuln: " + i['module'] + "\n"

         for j in i['data']:
            if i['request_type'] == 'GET':

               # For each (crawled) URL, print vulnerable URLs
               for l in j['data']:
                   output += "[GET] " + l + "\n"
               
            elif i['request_type'] == 'POST':
               output += "[POST]  " + j['url'][0] + "\n"
               
               # Print each vulnerable POST param...
               for p in j['data']:                   
                   k = p.keys()[0]
                   output += "\t" + k + ' = ' + p[k] + "\n"

         output += "\n"

                   
      #----- Print report
      if filename:
         fd = open(filename,"w")
         fd.write(output)
         fd.close()
      else:
         print output


# EOF
