#!/usr/bin/env python
#    uwss - uber web security scanner
#    Copyright (C) 2009 noptrix & illuminatus
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# uwss.py
#
# by noptrix & illuminatus
# www.noptrix.net


import sys
from optparse import OptionParser

# uwss imports
from uconfig.uController import *


def parse_cmdline_options():
   """
   parse the command lines (options and arguments)
   """
   
   parser = OptionParser(usage=usage, version=version)

   parser.add_option('-a', dest='attack_modus',
                    help='attack mode - ? for list')
   parser.add_option('-f', dest='filename', help='xml config file')
   parser.add_option('-c', action='store_true', dest='crawler',
                     help='crawl the webpage and save the data')
   parser.add_option('-l', action='store_true', dest='login',
                     help='authentificate to server and save cookies')
   parser.add_option('-m', dest='module', default='all',
                     help='attack module - ? for list')
   parser.add_option('-t', dest='request_type', default='all',
                     help='GET or POST - [default: %default]')
   parser.add_option('-r', default='stdout', dest='report',
                     help='report style - ? for list [default: %default]')
   parser.add_option('-q', action='store_false', dest='verbose', 
                     help='quite mode - only results [default]')
   parser.add_option('-v', action='store_true', dest='verbose',
                     help='verbose mode - print each attack url after report')
   parser.add_option('-d', action='store_true', dest='debug',
                     help='debug mode - print every action')
   
   (options, args) = parser.parse_args()

   return (options, args)


def main():
   """
   main routine
   """
   
   Controller.print_banner()

   (options, args) = parse_cmdline_options()

   if len(sys.argv) == 1:
      print '[-] WARNING: -h for help and usage'
      sys.exit(0)
   if options.attack_modus == '?':
      print ATTACK_MODES
      sys.exit(0)
   if options.report == '?':
      print REPORT_STYLES
      sys.exit(0)
   if options.module == '?':
      print ATTACK_MODULES
      sys.exit(0)

   # prepare uwss now
   my_cntrl = Controller((options, args))

   # get verbosity/debug mode
   my_cntrl.get_output_mode()

   # get xml config
   xml_config = my_cntrl.get_xml_config((options, args))

   # Before doing any stuff authentificate to server
   if options.login:
      my_cntrl.get_auth_cookies((options, args), xml_config)
      if not options.crawler:
         return
 
   # crawl the website
   if options.crawler:
      my_cntrl.start_crawler((options, args), xml_config)

   # get links and forms, then report results
   if options.module:
      links, forms = my_cntrl.get_links_and_forms(xml_config)
      report_data = my_cntrl.start_attack_modules((options, args), xml_config,\
            links, forms)
      my_cntrl.report_results((options, args), report_data)


if __name__ == '__main__':
   
   try:
      main()
   except KeyboardInterrupt:
      print "[-] aborted by user!"
   print '\n' + GOOD_BYE


# EOF
